ps_models = [
    "Empty",
    "FBP",
    "FBP_DCLink",
    "FAC_ACDC",
    "FAC_DCDC",
    "FAC_2S_ACDC",
    "FAC_2S_DCDC",
    "FAC_2P4S_ACDC",
    "FAC_2P4S_DCDC",
    "FAP",
    "FAP_4P",
    "FAC_DCDC_EMA",
    "FAP_2P2S",
    "FAP_IMAS",
    "FAC_2P_ACDC_IMAS",
    "FAC_2P_DCDC_IMAS",
    "SWLS_RESONANT_CONVERTER",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Invalid",
    "Uninitialized",
]

vars = [
    "ps_status",
    "ps_setpoint",
    "ps_reference",
    "firmware_version",
    "counter_set_slowref",
    "counter_sync_pulse",
    "siggen_enable",
    "siggen_type",
    "siggen_num_cycles",
    "siggen_n",
    "siggen_freq",
    "siggen_amplitude",
    "siggen_offset",
    "siggen_aux_param",
    "wfmref_selected",
    "wfmref_sync_mode",
    "wfmref_gain",
    "wfmref_offset",
    "p_wfmref_start",
    "p_wfmref_end",
    "p_wfmref_idx",
]

# curv = ['wfmref','buf_samples_ctom','buf_samples_mtoc']
curves = ["wfmref_data_0", "wfmref_data_1", "buf_samples_ctom"]

functions = [
    "turn_on",
    "turn_off",
    "open_loop",
    "closed_loop",
    "select_op_mode",
    "reset_interlocks",
    "set_command_interface",
    "set_serial_termination",
    "unlock_udc",
    "lock_udc",
    "cfg_source_scope",
    "cfg_freq_scope",
    "cfg_duration_scope",
    "enable_scope",
    "disable_scope",
    "sync_pulse",
    "set_slowref",
    "set_slowref_fbp",
    "set_slowref_readback_mon",
    "set_slowref_fbp_readback_mon",
    "set_slowref_readback_ref",
    "set_slowref_fbp_readback_ref",
    "reset_counters",
    "cfg_wfmref",
    "select_wfmref",
    "get_wfmref_size",
    "reset_wfmref",
    "cfg_siggen",
    "set_siggen",
    "enable_siggen",
    "disable_siggen",
    "set_param",
    "get_param",
    "save_param_eeprom",
    "load_param_eeprom",
    "save_param_bank",
    "load_param_bank",
    "set_dsp_coeffs",
    "get_dsp_coeff",
    "save_dsp_coeffs_eeprom",
    "load_dsp_coeffs_eeprom",
    "save_dsp_modules_eeprom",
    "load_dsp_modules_eeprom",
    "reset_udc",
]

op_modes = [
    "Off",
    "Interlock",
    "Initializing",
    "SlowRef",
    "SlowRefSync",
    "Cycle",
    "RmpWfm",
    "MigWfm",
    "FastRef",
]

sig_gen_types = [
    "Sine",
    "DampedSine",
    "Trapezoidal",
    "DampedSquaredSine",
    "Square",
]

wfmref_sync_modes = ["SampleBySample", "SampleBySample_OneCycle", "OneShot"]

versions = [
    "udc_arm",
    "udc_c28",
    "hradc0_cpld",
    "hradc1_cpld",
    "hradc2_cpld",
    "hradc3_cpld",
    "iib_arm",
    "ihm_pic",
]

params = {
    "PS_Name": {"id": 0, "n": 64},
    "PS_Model": {"id": 1, "n": 1},
    "Num_PS_Modules": {"id": 2, "n": 1},
    "Command_Interface": {"id": 3, "n": 1},
    "RS485_Baudrate": {"id": 4, "n": 1},
    "RS485_Address": {"id": 5, "n": 4},
    "RS485_Termination": {"id": 6, "n": 1},
    "UDCNet_Address": {"id": 7, "n": 1},
    "Ethernet_IP": {"id": 8, "n": 4},
    "Ethernet_Subnet_Mask": {"id": 9, "n": 4},
    "Buzzer_Volume": {"id": 10, "n": 1},
    "Freq_ISR_Controller": {"id": 11, "n": 1},
    "Freq_TimeSlicer": {"id": 12, "n": 4},
    "Control_Loop_State": {"id": 13, "n": 1},
    "Max_Ref": {"id": 14, "n": 4},
    "Min_Ref": {"id": 15, "n": 4},
    "Max_Ref_OpenLoop": {"id": 16, "n": 4},
    "Min_Ref_OpenLoop": {"id": 17, "n": 4},
    "PWM_Freq": {"id": 18, "n": 1},
    "PWM_DeadTime": {"id": 19, "n": 1},
    "PWM_Max_Duty": {"id": 20, "n": 1},
    "PWM_Min_Duty": {"id": 21, "n": 1},
    "PWM_Max_Duty_OpenLoop": {"id": 22, "n": 1},
    "PWM_Min_Duty_OpenLoop": {"id": 23, "n": 1},
    "PWM_Lim_Duty_Share": {"id": 24, "n": 1},
    "HRADC_Num_Boards": {"id": 25, "n": 1},
    "HRADC_Freq_SPICLK": {"id": 26, "n": 1},
    "HRADC_Freq_Sampling": {"id": 27, "n": 1},
    "HRADC_Enable_Heater": {"id": 28, "n": 4},
    "HRADC_Enable_Monitor": {"id": 29, "n": 4},
    "HRADC_Type_Transducer": {"id": 30, "n": 4},
    "HRADC_Gain_Transducer": {"id": 31, "n": 4},
    "HRADC_Offset_Transducer": {"id": 32, "n": 4},
    "SigGen_Type": {"id": 33, "n": 1},
    "SigGen_Num_Cycles": {"id": 34, "n": 1},
    "SigGen_Freq": {"id": 35, "n": 1},
    "SigGen_Amplitude": {"id": 36, "n": 1},
    "SigGen_Offset": {"id": 37, "n": 1},
    "SigGen_Aux_Param": {"id": 38, "n": 4},
    "WfmRef_ID_WfmRef": {"id": 39, "n": 4},
    "WfmRef_SyncMode": {"id": 40, "n": 4},
    "WfmRef_Frequency": {"id": 41, "n": 4},
    "WfmRef_Gain": {"id": 42, "n": 4},
    "WfmRef_Offset": {"id": 43, "n": 4},
    "Analog_Var_Max": {"id": 44, "n": 64},
    "Analog_Var_Min": {"id": 45, "n": 64},
    "Hard_Interlocks_Debounce_Time": {"id": 46, "n": 32},
    "Hard_Interlocks_Reset_Time": {"id": 47, "n": 32},
    "Soft_Interlocks_Debounce_Time": {"id": 48, "n": 32},
    "Soft_Interlocks_Reset_Time": {"id": 49, "n": 32},
    "Scope_Sampling_Frequency": {"id": 50, "n": 4},
    "Scope_Source": {"id": 51, "n": 4},
    "": {"id": 61, "n": 1},
    "Password": {"id": 62, "n": 4},
    "Enable_Onboard_EEPROM": {"id": 63, "n": 1},
}

bsmp = {
    "ps_status": {"addr": 0, "format": "H", "size": 2, "egu": ""},
    "ps_setpoint": {"addr": 1, "format": "f", "size": 4, "egu": "A"},
    "ps_reference": {"addr": 2, "format": "f", "size": 4, "egu": "A"},
    "firmware_version": {"addr": 3, "format": "128s", "size": 128, "egu": ""},
    "counter_set_slowref": {"addr": 4, "format": "I", "size": 4, "egu": ""},
    "counter_sync_pulse": {"addr": 5, "format": "I", "size": 4, "egu": ""},
    "siggen_enable": {"addr": 6, "format": "H", "size": 2, "egu": ""},
    "siggen_type": {"addr": 7, "format": "H", "size": 2, "egu": ""},
    "siggen_num_cycles": {"addr": 8, "format": "H", "size": 2, "egu": ""},
    "siggen_n": {"addr": 9, "format": "f", "size": 4, "egu": ""},
    "siggen_freq": {"addr": 10, "format": "f", "size": 4, "egu": "Hz"},
    "siggen_amplitude": {"addr": 11, "format": "f", "size": 4, "egu": "A"},
    "siggen_offset": {"addr": 12, "format": "f", "size": 4, "egu": "A"},
    "siggen_aux_param_0": {"addr": 13, "format": "f", "size": 4, "egu": ""},
    "siggen_aux_param_1": {"addr": 13, "format": "f", "size": 4, "egu": ""},
    "siggen_aux_param_2": {"addr": 13, "format": "f", "size": 4, "egu": ""},
    "siggen_aux_param_3": {"addr": 13, "format": "f", "size": 4, "egu": ""},
    "wfmref_selected": {"addr": 14, "format": "H", "size": 2, "egu": ""},
    "wfmref_sync_mode": {"addr": 15, "format": "H", "size": 2, "egu": ""},
    "wfmref_frequency": {"addr": 16, "format": "f", "size": 4, "egu": "Hz"},
    "wfmref_gain": {"addr": 17, "format": "f", "size": 4, "egu": "A"},
    "wfmref_offset": {"addr": 18, "format": "f", "size": 4, "egu": "A"},
    "p_wfmref_start_0": {"addr": 19, "format": "I", "size": 4, "egu": ""},
    "p_wfmref_end_0": {"addr": 20, "format": "I", "size": 4, "egu": ""},
    "p_wfmref_idx_0": {"addr": 21, "format": "I", "size": 4, "egu": ""},
    "p_wfmref_start_1": {"addr": 22, "format": "I", "size": 4, "egu": ""},
    "p_wfmref_end_1": {"addr": 23, "format": "I", "size": 4, "egu": ""},
    "p_wfmref_idx_1": {"addr": 24, "format": "I", "size": 4, "egu": ""},
    "scope_frequency": {"addr": 25, "format": "f", "size": 4, "egu": "Hz"},
    "scope_duration": {"addr": 26, "format": "f", "size": 4, "egu": "s"},
    "scope source_data": {"addr": 27, "format": "I", "size": 4, "egu": ""},
    "period_sync_pulse": {"addr": 28, "format": "I", "size": 4, "egu": ""},
}



# FBP
soft_interlocks = ["Heat-Sink Overtemperature"]

hard_interlocks = [
    "Load Overcurrent",
    "Load Overvoltage",
    "DCLink Overvoltage",
    "DCLink Undervoltage",
    "DCLink Relay Fault",
    "DCLink Fuse Fault",
    "MOSFETs Driver Fault",
    "Welded Relay Fault",
]

# FBP DC-Link
dclink_hard_interlocks = [
    "Power_Module_1_Fault",
    "Power_Module_2_Fault",
    "Power_Module_3_Fault",
    "Total_Output_Overvoltage",
    "Power_Module_1_Overvoltage",
    "Power_Module_2_Overvoltage",
    "Power_Module_3_Overvoltage",
    "Total_Output_Undervoltage",
    "Power_Module_1_Undervoltage",
    "Power_Module_2_Undervoltage",
    "Power_Module_3_Undervoltage",
    "Smoke_Detector",
    "External_Interlock",
]

bsmp = {
    "i_load": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "v_load": {"addr": 34, "format": "f", "size": 4, "egu": "V"},
    "v_dclink": {"addr": 35, "format": "f", "size": 4, "egu": "V"},
    "temp_switches": {"addr": 36, "format": "f", "size": 4, "egu": "°C"},
    "duty_cycle": {"addr": 37, "format": "f", "size": 4, "egu": "%"},
    "ps_alarms": {"addr": 38, "format": "I", "size": 4, "egu": ""},
    "reserved": {"addr": 39, "format": "7s", "size": 7, "egu": ""},
    "ps_status_1": {"addr": 46, "format": "H", "size": 2, "egu": ""},
    "ps_status_2": {"addr": 47, "format": "H", "size": 2, "egu": ""},
    "ps_status_3": {"addr": 48, "format": "H", "size": 2, "egu": ""},
    "ps_status_4": {"addr": 49, "format": "H", "size": 2, "egu": ""},
    "ps_setpoint_1": {"addr": 50, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_setpoint_2": {"addr": 51, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_setpoint_3": {"addr": 52, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_setpoint_4": {"addr": 53, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_reference_1": {"addr": 54, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_reference_2": {"addr": 55, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_reference_3": {"addr": 56, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_reference_4": {"addr": 57, "format": "f", "size": 4, "egu": "A / V / %"},
    "ps_soft_interlocks_1": {"addr": 58, "format": "I", "size": 4, "egu": ""},
    "ps_soft_interlocks_2": {"addr": 59, "format": "I", "size": 4, "egu": ""},
    "ps_soft_interlocks_3": {"addr": 60, "format": "I", "size": 4, "egu": ""},
    "ps_soft_interlocks_4": {"addr": 61, "format": "I", "size": 4, "egu": ""},
    "ps_hard_interlocks_1": {"addr": 62, "format": "I", "size": 4, "egu": ""},
    "ps_hard_interlocks_2": {"addr": 63, "format": "I", "size": 4, "egu": ""},
    "ps_hard_interlocks_3": {"addr": 64, "format": "I", "size": 4, "egu": ""},
    "ps_hard_interlocks_4": {"addr": 65, "format": "I", "size": 4, "egu": ""},
    "i_load_1": {"addr": 66, "format": "f", "size": 4, "egu": "A"},
    "i_load_2": {"addr": 67, "format": "f", "size": 4, "egu": "A"},
    "i_load_3": {"addr": 68, "format": "f", "size": 4, "egu": "A"},
    "i_load_4": {"addr": 69, "format": "f", "size": 4, "egu": "A"},
    "ps_alarms_1": {"addr": 70, "format": "I", "size": 4, "egu": ""},
    "ps_alarms_2": {"addr": 71, "format": "I", "size": 4, "egu": ""},
    "ps_alarms_3": {"addr": 72, "format": "I", "size": 4, "egu": ""},
    "ps_alarms_4": {"addr": 73, "format": "I", "size": 4, "egu": ""},
}

bsmp_dclink = {
    "modules_status": {"addr": 33, "format": "I", "size": 4, "egu": ""},
    "v_out": {"addr": 34, "format": "f", "size": 4, "egu": "V"},
    "v_out_1": {"addr": 35, "format": "f", "size": 4, "egu": "V"},
    "v_out_2": {"addr": 36, "format": "f", "size": 4, "egu": "V"},
    "v_out_3": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "dig_pot_tap": {"addr": 38, "format": "B", "size": 1, "egu": "%"},
}



"""Utility functions"""
import logging as _logging
import struct
import typing as _typing


def get_logger(
    name=__file__,
    level: int = _logging.INFO,
    handlers: _typing.Optional[_typing.List[_logging.Handler]] = None,
) -> _logging.Logger:
    """Returns a logger object"""

    logger = _logging.getLogger(name)

    if not logger.handlers and not handlers:
        formatter = _logging.Formatter(
            "[%(asctime)s %(levelname)s %(filename)s:%(lineno)s - %(funcName)s] %(message)s"
        )
        logger.setLevel(level)
        console = _logging.StreamHandler()
        console.setFormatter(formatter)
        logger.addHandler(console)
    return logger


def float_to_hex(value: float):
    hex_value = struct.pack("f", value)
    return hex_value.decode("ISO-8859-1")


def float_list_to_hex(value_list: list):
    hex_list = b""
    for value in value_list:
        hex_list = hex_list + struct.pack("f", value)
    return hex_list.decode("ISO-8859-1")


def format_list_size(in_list: list, max_size: int):
    out_list = in_list[0:max_size]
    if max_size > len(in_list):
        for _ in range(max_size - len(in_list)):
            out_list.append(0)
    return out_list


def double_to_hex(value: int):
    hex_value = struct.pack("H", value)
    return hex_value.decode("ISO-8859-1")


def uint32_to_hex(value: int):
    hex_value = struct.pack("I", value)
    return hex_value.decode("ISO-8859-1")


def index_to_hex(value: int):
    hex_value = struct.pack("B", value)
    return hex_value.decode("ISO-8859-1")


def size_to_hex(value: int):
    hex_value = struct.pack(">H", value)
    return hex_value.decode("ISO-8859-1")


def checksum(packet: bytes) -> bytes:
    csum = (256 - sum(packet)) % 256
    return packet + bytes([csum])


def prettier_print(var_input: dict, prefix: str = ""):
    for key, value in var_input.items():
        if isinstance(value, dict):
            prettier_print(value, "".join([prefix, key.replace("_", " ").upper(), " "]))
        else:
            key_words = key.split("_")

            for i in range(0, len(key_words)):
                key_words[i] = (
                    key_words[i].upper()
                    if key_words[i] in ["igbt", "dc", "iib", "ps", "ip", "idb"]
                    else key_words[i].capitalize()
                )
            print(f"{prefix}{' '.join(key_words)}: {value}")



# from time import time
import typing

import serial
import serial.serialutil
from siriuspy.bsmp.serial import IOInterface as _IOInterface

from pydrs.bsmp.exceptions import create_value_error


class TCPInterface(_IOInterface):
    pass


class SerialInterface(_IOInterface):
    def __init__(
        self,
        path: str,
        baudrate: int,
        auto_connect: bool = True,
        encoding: str = "utf-8",
    ):
        super().__init__()

        if not path or type(path) != str:
            create_value_error(parameter="path", input=path)

        if (not baudrate) or type(baudrate) != int or baudrate < 0:
            create_value_error(parameter="baudrate", input=baudrate)

        self._port: str = path
        self._baudrate: int = baudrate
        self._serial: typing.Optional[serial.serialutil.SerialBase] = None
        self._encoding: str = encoding

        if auto_connect:
            self.open()

    def __enter__(self):
        self.open()
        return self

    def __exit__(self, *args, **kwargs):
        if self._serial and self._serial.is_open:
            self._serial.close()

    @property
    def encoding(self) -> str:
        return self._encoding

    def open(self) -> None:
        """Open the serial connection"""
        if not self._serial:
            self._serial = serial.Serial(port=self._port, baudrate=self._baudrate)

    def close(self) -> None:
        """Close the serial connection"""
        if self._serial:
            self._serial.close()
            self._serial = None

    def UART_read(self) -> typing.List[str]:
        # @todo: Usar a especificação do bsmp para ler o número correto de bytes
        if not self._serial:
            raise Exception("Serial not defined")

        _response = self._serial.read_all()
        _decoded = _response.decode(self._encoding)
        return [s for s in _decoded]

    def UART_write(
        self, stream: typing.List[str], timeout: float
    ) -> typing.Optional[typing.Any]:
        if not self._serial:
            raise Exception("Serial not defined")

        return self._serial.write(stream)

    def UART_request(
        self, stream: typing.List[str], timeout: float
    ) -> typing.Optional[typing.List[str]]:
        """Read and Write OP"""
        self.UART_write(stream, timeout=timeout)
        return self.UART_read()



# SWLS Resonant Converter
list_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
]

list_hard_interlocks = [
    "Load Overcurrent",
    "DCLink Overvoltage",
    "DCLink Undervoltage",
    "Welded Contactor Fault",
    "Opened Contactor Fault",
    "External Interlock",
    "IIB Interlock"
]

list_iib_interlocks = [
    "Input Overvoltage",
    "Output Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "Transformer Heat-Sink Overtemperature",
    "Output Inductor Overtemperature",
    "Diode One Heat-Sink Overtemperature",
    "Diode Two Heat-Sink Overtemperature",
    "Driver MOSFETs and Auxiliary Board Overvoltage",
    "Driver MOSFETs Overcurrent",
    "Auxiliary Board Overcurrent",
    "High Leakage Current",
    "Board IIB Overtemperature",
    "Module Overhumidity",
    "Contact Sticking of Contactor",
    "Emergency Button",
]

list_iib_alarms = [
    "Input Overvoltage",
    "Output Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "Transformer Heat-Sink Overtemperature",
    "Output Inductor Overtemperature",
    "Diode One Heat-Sink Overtemperature",
    "Diode Two Heat-Sink Overtemperature",
    "Driver MOSFETs and Auxiliary Board Overvoltage",
    "Driver MOSFETs Overcurrent",
    "Auxiliary Board Overcurrent",
    "High Leakage Current",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

bsmp = {
    "ps_alarms": {"addr": 33, "format": "I", "size": 4, "egu": ""},
    "i_load_mean": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load_1": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "i_load_2": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "i_load_error": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "freq_modulated": {"addr": 38, "format": "f", "size": 4, "egu": "Hz"},
    "v_input_iib": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib": {"addr": 40, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib": {"addr": 41, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib": {"addr": 42, "format": "f", "size": 4, "egu": "A"},
    "temp_heatsink_transformer_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "temp_output_inductor_iib": {"addr": 44, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_diode_one_iib": {"addr": 45, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_diode_two_iib": {"addr": 46, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_and_aux_board_iib": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "i_driver_iib": {"addr": 48, "format": "f", "size": 4, "egu": "A"},
    "i_aux_board_iib": {"addr": 49, "format": "f", "size": 4, "egu": "A"},
    "i_leakage_iib": {"addr": 50, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib": {"addr": 51, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib": {"addr": 52, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks": {"addr": 53, "format": "I", "size": 4, "egu": ""},
    "iib_alarms": {"addr": 54, "format": "I", "size": 4, "egu": ""},
}


import typing

"""BSMP PS."""


def create_value_error(parameter: str, input: typing.Any):
    raise ValueError(f"Invalid value for parameter '{parameter}', received '{input}'")


class BasePSAckError(Exception):
    """Exception raised when the ack response is not the expected"""

    def __init__(self, message, *args: object) -> None:
        super().__init__(*args)


class FunctionExecutionPSAckError(BasePSAckError):
    """."""


class InvalidCommandPSAckError(BasePSAckError):
    """."""


class DSPBusyPSAckError(BasePSAckError):
    """."""


class DSPTimeoutPSAckError(BasePSAckError):
    """."""


class ResourceBusyPSAckError(BasePSAckError):
    """."""


class UDCLockedPSAckError(BasePSAckError):
    """."""


class PSInterlockPSAckError(BasePSAckError):
    """."""



# FAC ACDC
list_acdc_soft_interlocks = []

list_acdc_hard_interlocks = [
    "CapBank Overvoltage",
    "Rectifier Overvoltage",
    "Rectifier Undervoltage",
    "Rectifier Overcurrent",
    "Welded Contactor Fault",
    "Opened Contactor Fault",
    "IIB Input Stage Interlock",
    "IIB Command Interlock",
]

list_acdc_iib_is_interlocks = [
    "Rectifier Overvoltage",
    "Input Overcurrent",
    "IGBT Overtemperature",
    "IGBT Overtemperature HW",
    "Driver Overvoltage",
    "Driver Overcurrent",
    "Top Driver Error",
    "Bottom Driver Error",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_acdc_iib_is_alarms = [
    "Rectifier Overvoltage",
    "Input Overcurrent",
    "IGBT Overtemperature",
    "Driver Overvoltage",
    "Driver Overcurrent",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_acdc_iib_cmd_interlocks = [
    "Capbank Overvoltage",
    "Output Overvoltage",
    "External Boards Overvoltage",
    "Auxiliary Board Overcurrent",
    "IDB Board Overcurrent",
    "Rectifier Inductor Overtemperature",
    "Rectifier Heat-Sink Overtemperature",
    "AC Mains Overcurrent",
    "Emergency Button",
    "AC Mains Undervoltage",
    "AC Mains Overvoltage",
    "Ground Leakage Overcurrent",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_acdc_iib_cmd_alarms = [
    "Capbank Overvoltage",
    "Output Overvoltage",
    "External Boards Overvoltage",
    "Auxiliary Board Overcurrent",
    "IDB Board Overcurrent",
    "Rectifier Inductor Overtemperature",
    "Rectifier Heat-Sink Overtemperature",
    "Ground Leakage Overcurrent",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

# FAC DCDC
list_dcdc_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
]

list_dcdc_hard_interlocks = [
    "Load Overcurrent",
    "CapBank Overvoltage",
    "CapBank Undervoltage",
    "IIB Interlock",
    "External Interlock",
    "Rack Interlock",
    "Leakage_Overcurrent",
]

list_dcdc_iib_interlocks = [
    "Input Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 1 Overtemperature HW",
    "IGBT 2 Overtemperature",
    "IGBT 2 Overtemperature HW",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Top Driver 1 Error",
    "Bottom Driver 1 Error",
    "Top Driver 2 Error",
    "Bottom Driver 2 Error",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_dcdc_iib_alarms = [
    "Input Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 2 Overtemperature",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

# FAC-2S AC/DC
list_2s_acdc_soft_interlocks = []
list_2s_acdc_hard_interlocks = [
    "CapBank Overvoltage",
    "Rectifier Overvoltage",
    "Rectifier Undervoltage",
    "Rectifier Overcurrent",
    "Welded Contactor Fault",
    "Opened Contactor Fault",
    "IIB Input Stage Interlock",
    "IIB Command Interlock",
]

list_2s_acdc_iib_is_interlocks = list_acdc_iib_is_interlocks
list_2s_acdc_iib_cmd_interlocks = list_acdc_iib_cmd_interlocks
list_2s_acdc_iib_is_alarms = list_acdc_iib_is_alarms
list_2s_acdc_iib_cmd_alarms = list_acdc_iib_cmd_alarms

# FAC-2S DC/DC
list_2s_dcdc_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
]

list_2s_dcdc_hard_interlocks = [
    "Load Overcurrent",
    "Module 1 CapBank Overvoltage",
    "Module 2 CapBank Overvoltage",
    "Module 1 CapBank Undervoltage",
    "Module 2 CapBank Undervoltage",
    "IIB Mod 1 Itlk",
    "IIB Mod 2 Itlk",
    "External Interlock",
    "Rack Interlock",
]

list_2s_dcdc_iib_interlocks = list_dcdc_iib_interlocks
list_2s_dcdc_iib_alarms = list_dcdc_iib_alarms

# FAC-2P4S AC/DC
list_2p4s_acdc_hard_interlocks = [
    "CapBank Overvoltage",
    "Rectifier Overvoltage",
    "Rectifier Undervoltage",
    "Rectifier Overcurrent",
    "Welded Contactor Fault",
    "Opened Contactor Fault",
    "IIB Input Stage Interlock",
    "IIB Command Interlock",
]

list_2p4s_acdc_iib_is_interlocks = list_acdc_iib_is_interlocks
list_2p4s_acdc_iib_cmd_interlocks = list_acdc_iib_cmd_interlocks
list_2p4s_acdc_iib_is_alarms = list_acdc_iib_is_alarms
list_2p4s_acdc_iib_cmd_alarms = list_acdc_iib_cmd_alarms

# FAC-2P4S DC/DC
list_2p4s_dcdc_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
    "Arm 1 Overcurrent",
    "Arm 2 Overcurrent",
    "Arms High Difference",
    "Complementary PS Interlock",
]

list_2p4s_dcdc_hard_interlocks = [
    "Load Overcurrent",
    "Module 1 CapBank Overvoltage",
    "Module 2 CapBank Overvoltage",
    "Module 3 CapBank Overvoltage",
    "Module 4 CapBank Overvoltage",
    "Module 5 CapBank Overvoltage",
    "Module 6 CapBank Overvoltage",
    "Module 7 CapBank Overvoltage",
    "Module 8 CapBank Overvoltage",
    "Module 1 CapBank Undervoltage",
    "Module 2 CapBank Undervoltage",
    "Module 3 CapBank Undervoltage",
    "Module 4 CapBank Undervoltage",
    "Module 5 CapBank Undervoltage",
    "Module 6 CapBank Undervoltage",
    "Module 7 CapBank Undervoltage",
    "Module 8 CapBank Undervoltage",
    "IIB 1 Itlk",
    "IIB 2 Itlk",
    "IIB 3 Itlk",
    "IIB 4 Itlk",
    "IIB 5 Itlk",
    "IIB 6 Itlk",
    "IIB 7 Itlk",
    "IIB 8 Itlk",
]

list_2p4s_dcdc_iib_interlocks = list_dcdc_iib_interlocks
list_2p4s_dcdc_iib_alarms = list_dcdc_iib_alarms

# FAC DCDC EMA
list_dcdc_ema_soft_interlocks = ["DCCT Fault", "Load Feedback Fault"]

list_dcdc_ema_hard_interlocks = [
    "Load Overcurrent",
    "DCLink Overvoltage",
    "DCLink Undervoltage",
    "Emergency Button",
    "Load Waterflow",
    "Load Overtemperature",
    "IIB Itlk",
    "Leakage_Overcurrent",
]

list_dcdc_ema_iib_interlocks = [
    "Input Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 1 Overtemperature HW",
    "IGBT 2 Overtemperature",
    "IGBT 2 Overtemperature HW",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Top Driver 1 Error",
    "Bottom Driver 1 Error",
    "Top Driver 2 Error",
    "Bottom Driver 2 Error",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_dcdc_ema_iib_alarms = [
    "Input Overvoltage",
    "Input Overcurrent",
    "Output Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 2 Overtemperature",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

# FAC-2P ACDC
list_2p_acdc_imas_soft_interlocks = []

list_2p_acdc_imas_hard_interlocks = [
    "CapBank Overvoltage",
    "Rectifier Overcurrent",
    "AC Mains Contactor Fault",
    "Module A Interlock",
    "Module B Interlock",
    "DCDC Interlock",
]

# FAC-2P DCDC
list_2p_dcdc_imas_soft_interlocks = []

list_2p_dcdc_imas_hard_interlocks = [
    "Load Overcurrent",
    "Module 1 CapBank_Overvoltage",
    "Module 2 CapBank_Overvoltage",
    "Module 1 CapBank_Undervoltage",
    "Module 2 CapBank_Undervoltage",
    "Arm 1 Overcurrent",
    "Arm 2 Overcurrent",
    "Arms High_Difference",
    "ACDC Interlock",
]

bsmp_acdc = {
    "v_capacitor_bank": {"addr": 33, "format": "f", "size": 4, "egu": "V"},
    "i_out_rectifier": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "duty_cycle": {"addr": 35, "format": "f", "size": 4, "egu": "p.u."},
    "i_input_is_iib": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "v_input_is_iib": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "temp_igbt_is_iib": {"addr": 38, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_is_iib": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "i_driver_is_iib": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_is_iib": {"addr": 41, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_is_iib": {"addr": 42, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_is_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "rh_is_iib": {"addr": 44, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_is": {"addr": 45, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_is": {"addr": 46, "format": "I", "size": 4, "egu": ""},
    "v_output_cmd_iib": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_cmd_iib": {"addr": 48, "format": "f", "size": 4, "egu": "V"},
    "temp_rect_inductor_cmd_iib": {"addr": 49, "format": "f", "size": 4, "egu": "°C"},
    "temp_rect_heatsink_cmd_iib": {"addr": 50, "format": "f", "size": 4, "egu": "°C"},
    "v_ext_boards_cmd_iib": {"addr": 51, "format": "f", "size": 4, "egu": "V"},
    "i_aux_board_cmd_iib": {"addr": 52, "format": "f", "size": 4, "egu": "A"},
    "i_idb_board_cmd_iib": {"addr": 53, "format": "f", "size": 4, "egu": "A"},
    "i_leakage_cmd_iib": {"addr": 54, "format": "f", "size": 4, "egu": "A"},
    "temp_board_cmd_iib": {"addr": 55, "format": "f", "size": 4, "egu": "°C"},
    "rh_cmd_iib": {"addr": 56, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_cmd": {"addr": 57, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_cmd": {"addr": 58, "format": "I", "size": 4, "egu": ""},
}

bsmp_dcdc = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "v_capacitor_bank": {"addr": 36, "format": "f", "size": 4, "egu": "V"},
    "duty_cycle": {"addr": 37, "format": "f", "size": 4, "egu": "p.u."},
    "i_leakage": {"addr": 38, "format": "f", "size": 4, "egu": "A"},
    "v_input_iib": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib": {"addr": 41, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib": {"addr": 42, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib": {"addr": 44, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib": {"addr": 45, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib": {"addr": 46, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib": {"addr": 47, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib": {"addr": 48, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib": {"addr": 49, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib": {"addr": 50, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks": {"addr": 51, "format": "I", "size": 4, "egu": ""},
    "iib_alarms": {"addr": 52, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 53, "format": "I", "size": 4, "egu": ""},
}

bsmp_dcdc_ema = {
    "i_load": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "v_dclink": {"addr": 34, "format": "f", "size": 4, "egu": "V"},
    "duty_cycle": {"addr": 35, "format": "f", "size": 4, "egu": "p.u."},
    "i_leakage": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "v_input_iib": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib": {"addr": 38, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib": {"addr": 39, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib": {"addr": 40, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib": {"addr": 41, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib": {"addr": 42, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib": {"addr": 44, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib": {"addr": 45, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib": {"addr": 46, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib": {"addr": 47, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib": {"addr": 48, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks": {"addr": 49, "format": "I", "size": 4, "egu": ""},
    "iib_alarms": {"addr": 50, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 51, "format": "I", "size": 4, "egu": ""},
}

bsmp_2s_acdc = {
    "v_capacitor_bank": {"addr": 33, "format": "f", "size": 4, "egu": "V"},
    "i_out_rectifier": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "duty_cycle": {"addr": 35, "format": "f", "size": 4, "egu": "p.u."},
    "i_input_is_iib": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "v_input_is_iib": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "temp_igbt_is_iib": {"addr": 38, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_is_iib": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "i_driver_is_iib": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_is_iib": {"addr": 41, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_is_iib": {"addr": 42, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_is_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "rh_is_iib": {"addr": 44, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_is": {"addr": 45, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_is": {"addr": 46, "format": "I", "size": 4, "egu": ""},
    "v_output_cmd_iib": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_cmd_iib": {"addr": 48, "format": "f", "size": 4, "egu": "V"},
    "temp_rect_inductor_cmd_iib": {"addr": 49, "format": "f", "size": 4, "egu": "°C"},
    "temp_rect_heatsink_cmd_iib": {"addr": 50, "format": "f", "size": 4, "egu": "°C"},
    "v_ext_boards_cmd_iib": {"addr": 51, "format": "f", "size": 4, "egu": "V"},
    "i_aux_board_cmd_iib": {"addr": 52, "format": "f", "size": 4, "egu": "A"},
    "i_idb_board_cmd_iib": {"addr": 53, "format": "f", "size": 4, "egu": "A"},
    "i_leakage_cmd_iib": {"addr": 54, "format": "f", "size": 4, "egu": "A"},
    "temp_board_cmd_iib": {"addr": 55, "format": "f", "size": 4, "egu": "°C"},
    "rh_cmd_iib": {"addr": 56, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_cmd": {"addr": 57, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_cmd": {"addr": 58, "format": "I", "size": 4, "egu": ""},
}

bsmp_2s_dcdc = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "v_capbank_1": {"addr": 36, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_2": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "duty_cycle_1": {"addr": 38, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2": {"addr": 39, "format": "f", "size": 4, "egu": "p.u."},
    "v_input_iib_1": {"addr": 40, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib_1": {"addr": 41, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib_1": {"addr": 42, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib_1": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib_1": {"addr": 44, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib_1": {"addr": 45, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_1": {"addr": 46, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_1": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_1": {"addr": 48, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_1": {"addr": 49, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib_1": {"addr": 50, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_1": {"addr": 51, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_1": {"addr": 52, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_1": {"addr": 53, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_2": {"addr": 54, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib_2": {"addr": 55, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib_2": {"addr": 56, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib_2": {"addr": 57, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib_2": {"addr": 58, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib_2": {"addr": 59, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_2": {"addr": 60, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_2": {"addr": 61, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_2": {"addr": 62, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_2": {"addr": 63, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib_2": {"addr": 64, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_2": {"addr": 65, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_2": {"addr": 66, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_2": {"addr": 67, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 68, "format": "I", "size": 4, "egu": ""},
}

bsmp_2p_acdc_imas = {
    "v_capacitor_bank": {"addr": 33, "format": "f", "size": 4, "egu": "V"},
    "i_out_rectifier": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "duty_cycle": {"addr": 35, "format": "f", "size": 4, "egu": "p.u."},
}

bsmp_2p_dcdc_imas = {
    "i_load": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load_error": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_arm_1": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "i_arm_2": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "i_arms_diff": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "v_capbank_1": {"addr": 38, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_2": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "duty_cycle_1": {"addr": 40, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2": {"addr": 41, "format": "f", "size": 4, "egu": "p.u."},
    "duty_diff": {"addr": 42, "format": "f", "size": 4, "egu": "p.u."},
}

bsmp_2p4s_acdc = {
    "v_capacitor_bank": {"addr": 33, "format": "f", "size": 4, "egu": "V"},
    "i_out_rectifier": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "duty_cycle": {"addr": 35, "format": "f", "size": 4, "egu": "p.u."},
    "i_input_is_iib": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "v_input_is_iib": {"addr": 37, "format": "f", "size": 4, "egu": "V"},
    "temp_igbt_is_iib": {"addr": 38, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_is_iib": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "i_driver_is_iib": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_is_iib": {"addr": 41, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_is_iib": {"addr": 42, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_is_iib": {"addr": 43, "format": "f", "size": 4, "egu": "°C"},
    "rh_is_iib": {"addr": 44, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_is": {"addr": 45, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_is": {"addr": 46, "format": "I", "size": 4, "egu": ""},
    "v_output_cmd_iib": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_cmd_iib": {"addr": 48, "format": "f", "size": 4, "egu": "V"},
    "temp_rect_inductor_cmd_iib": {"addr": 49, "format": "f", "size": 4, "egu": "°C"},
    "temp_rect_heatsink_cmd_iib": {"addr": 50, "format": "f", "size": 4, "egu": "°C"},
    "v_ext_boards_cmd_iib": {"addr": 51, "format": "f", "size": 4, "egu": "V"},
    "i_aux_board_cmd_iib": {"addr": 52, "format": "f", "size": 4, "egu": "A"},
    "i_idb_board_cmd_iib": {"addr": 53, "format": "f", "size": 4, "egu": "A"},
    "i_leakage_cmd_iib": {"addr": 54, "format": "f", "size": 4, "egu": "A"},
    "temp_board_cmd_iib": {"addr": 55, "format": "f", "size": 4, "egu": "°C"},
    "rh_cmd_iib": {"addr": 56, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_cmd": {"addr": 57, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_cmd": {"addr": 58, "format": "I", "size": 4, "egu": ""},
}

bsmp_2p4s_dcdc = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "i_arm_1": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "i_arm_2": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "v_capbank_1": {"addr": 38, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_2": {"addr": 39, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_3": {"addr": 40, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_4": {"addr": 41, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_5": {"addr": 42, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_6": {"addr": 43, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_7": {"addr": 44, "format": "f", "size": 4, "egu": "V"},
    "v_capbank_8": {"addr": 45, "format": "f", "size": 4, "egu": "V"},
    "duty_cycle_1": {"addr": 46, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2": {"addr": 47, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_3": {"addr": 48, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_4": {"addr": 49, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_5": {"addr": 50, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_6": {"addr": 51, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_7": {"addr": 52, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_8": {"addr": 53, "format": "f", "size": 4, "egu": "p.u."},
    "v_input_iib_a": {"addr": 54, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib_a": {"addr": 55, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib_a": {"addr": 56, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib_a": {"addr": 57, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib_a": {"addr": 58, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib_a": {"addr": 59, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_a": {"addr": 60, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_a": {"addr": 61, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_a": {"addr": 62, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_a": {"addr": 63, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib_a": {"addr": 64, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_a": {"addr": 65, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_a": {"addr": 66, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_a": {"addr": 67, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_b": {"addr": 68, "format": "f", "size": 4, "egu": "V"},
    "i_input_iib_b": {"addr": 69, "format": "f", "size": 4, "egu": "A"},
    "i_output_iib_b": {"addr": 70, "format": "f", "size": 4, "egu": "A"},
    "temp_igbts_1_iib_b": {"addr": 71, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbts_2_iib_b": {"addr": 72, "format": "f", "size": 4, "egu": "°C"},
    "temp_inductor_iib_b": {"addr": 73, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_b": {"addr": 74, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_b": {"addr": 75, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_b": {"addr": 76, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_b": {"addr": 77, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib_b": {"addr": 78, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_b": {"addr": 79, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_b": {"addr": 80, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_b": {"addr": 81, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 82, "format": "I", "size": 4, "egu": ""},
}



# FAP
list_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
    "IGBTs Current High Difference",
]

list_hard_interlocks = [
    "Load Overcurrent",
    "Load Overvoltage",
    "DCLink Overvoltage",
    "DCLink Undervoltage",
    "Welded Contactor Fault",
    "Opened Contactor Fault",
    "IGBT 1 Overcurrent",
    "IGBT 2 Overcurrent",
    "IIB Itlk",
]

list_iib_interlocks = [
    "Input Overvoltage",
    "Output Overvoltage",
    "IGBT 1 Overcurrent",
    "IGBT 2 Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 2 Overtemperature",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Driver 1 Error",
    "Driver 2 Error",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "DCLink Contactor Fault",
    "Contact Sticking of Contactor",
    "External Interlock",
    "Rack Interlock",
    "High Leakage Current",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

list_iib_alarms = [
    "Input Overvoltage",
    "Output Overvoltage",
    "IGBT 1 Overcurrent",
    "IGBT 2 Overcurrent",
    "IGBT 1 Overtemperature",
    "IGBT 2 Overtemperature",
    "Driver Overvoltage",
    "Driver 1 Overcurrent",
    "Driver 2 Overcurrent",
    "Inductors Overtemperature",
    "Heat-Sink Overtemperature",
    "High Leakage Current",
    "Board IIB Overtemperature",
    "Module Overhumidity",
]

# FAP-4P
list_4p_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
    "IGBTs Current High Difference",
]

list_4p_hard_interlocks = [
    "Load Overcurrent",
    "Load Overvoltage",
    "IGBT 1 Mod 1 Overcurrent",
    "IGBT 2 Mod 1 Overcurrent",
    "IGBT 1 Mod 2 Overcurrent",
    "IGBT 2 Mod 2 Overcurrent",
    "IGBT 1 Mod 3 Overcurrent",
    "IGBT 2 Mod 3 Overcurrent",
    "IGBT 1 Mod 4 Overcurrent",
    "IGBT 2 Mod 4 Overcurrent",
    "Welded Contactor Mod 1 Fault",
    "Welded Contactor Mod 2 Fault",
    "Welded Contactor Mod 3 Fault",
    "Welded Contactor Mod 4 Fault",
    "Opened Contactor Mod 1 Fault",
    "Opened Contactor Mod 2 Fault",
    "Opened Contactor Mod 3 Fault",
    "Opened Contactor Mod 4 Fault",
    "DCLink Mod 1 Overvoltage",
    "DCLink Mod 2 Overvoltage",
    "DCLink Mod 3 Overvoltage",
    "DCLink Mod 4 Overvoltage",
    "DCLink Mod 1 Undervoltage",
    "DCLink Mod 2 Undervoltage",
    "DCLink Mod 3 Undervoltage",
    "DCLink Mod 4 Undervoltage",
    "IIB Mod 1 Itlk",
    "IIB Mod 2 Itlk",
    "IIB Mod 3 Itlk",
    "IIB Mod 4 Itlk",
]

list_4p_iib_interlocks = list_iib_interlocks
list_4p_iib_alarms = list_iib_alarms

# FAP-2P2S
list_2p2s_soft_interlocks = [
    "DCCT 1 Fault",
    "DCCT 2 Fault",
    "DCCT High Difference",
    "Load Feedback 1 Fault",
    "Load Feedback 2 Fault",
    "Arms High Difference",
    "IGBTs Current High Difference",
    "Complementary PS Interlock",
]

list_2p2s_hard_interlocks = [
    "Load Overcurrent",
    "IGBT 1 Mod 1 Overcurrent",
    "IGBT 2 Mod 1 Overcurrent",
    "IGBT 1 Mod 2 Overcurrent",
    "IGBT 2 Mod 2 Overcurrent",
    "IGBT 1 Mod 3 Overcurrent",
    "IGBT 2 Mod 3 Overcurrent",
    "IGBT 1 Mod 4 Overcurrent",
    "IGBT 2 Mod 4 Overcurrent",
    "Welded Contactor Mod 1 Fault",
    "Welded Contactor Mod 2 Fault",
    "Welded Contactor Mod 3 Fault",
    "Welded Contactor Mod 4 Fault",
    "Opened Contactor Mod 1 Fault",
    "Opened Contactor Mod 2 Fault",
    "Opened Contactor Mod 3 Fault",
    "Opened Contactor Mod 4 Fault",
    "DCLink Mod 1 Overvoltage",
    "DCLink Mod 2 Overvoltage",
    "DCLink Mod 3 Overvoltage",
    "DCLink Mod 4 Overvoltage",
    "DCLink Mod 1 Undervoltage",
    "DCLink Mod 2 Undervoltage",
    "DCLink Mod 3 Undervoltage",
    "DCLink Mod 4 Undervoltage",
    "IIB Mod 1 Itlk",
    "IIB Mod 2 Itlk",
    "IIB Mod 3 Itlk",
    "IIB Mod 4 Itlk",
    "Arm 1 Overcurrent",
    "Arm 2 Overcurrent",
]

list_2p2s_iib_interlocks = list_iib_interlocks
list_2p2s_iib_alarms = list_iib_alarms

# FAP 225A
list_225A_soft_interlocks = ["IGBTs Current High Difference"]

list_225A_hard_interlocks = [
    "Load Overcurrent",
    "DCLink Contactor Fault",
    "IGBT 1 Overcurrent",
    "IGBT 2 Overcurrent",
]

bsmp = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "v_dclink": {"addr": 36, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2": {"addr": 38, "format": "f", "size": 4, "egu": "A"},
    "duty_cycle_1": {"addr": 39, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2": {"addr": 40, "format": "f", "size": 4, "egu": "p.u."},
    "duty_diff": {"addr": 41, "format": "f", "size": 4, "egu": "p.u."},
    "v_input_iib": {"addr": 42, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib": {"addr": 43, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib": {"addr": 44, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib": {"addr": 45, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib": {"addr": 46, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib": {"addr": 47, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib": {"addr": 48, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib": {"addr": 49, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib": {"addr": 50, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib": {"addr": 51, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib": {"addr": 52, "format": "f", "size": 4, "egu": "°C"},
    "i_leakage_iib": {"addr": 53, "format": "f", "size": 4, "egu": "A"},
    "temp_board_iib": {"addr": 54, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib": {"addr": 55, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks": {"addr": 56, "format": "I", "size": 4, "egu": ""},
    "iib_alarms": {"addr": 57, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 58, "format": "I", "size": 4, "egu": ""},
}

bsmp_4p = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "v_load": {"addr": 36, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_1": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_1": {"addr": 38, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_2": {"addr": 39, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_2": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_3": {"addr": 41, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_3": {"addr": 42, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_4": {"addr": 43, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_4": {"addr": 44, "format": "f", "size": 4, "egu": "A"},
    "v_dclink_1": {"addr": 45, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_2": {"addr": 46, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_3": {"addr": 47, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_4": {"addr": 48, "format": "f", "size": 4, "egu": "V"},
    "duty_mean": {"addr": 49, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_1": {"addr": 50, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_1": {"addr": 51, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_2": {"addr": 52, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_2": {"addr": 53, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_3": {"addr": 54, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_3": {"addr": 55, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_4": {"addr": 56, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_4": {"addr": 57, "format": "f", "size": 4, "egu": "p.u."},
    "i_leakage": {"addr": 58, "format": "f", "size": 4, "egu": "A"},
    "v_input_iib_1": {"addr": 59, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_1": {"addr": 60, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_1": {"addr": 61, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_1": {"addr": 62, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_1": {"addr": 63, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_1": {"addr": 64, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_1": {"addr": 65, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_1": {"addr": 66, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_1": {"addr": 67, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_1": {"addr": 68, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_1": {"addr": 69, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_1": {"addr": 70, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_1": {"addr": 71, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_1": {"addr": 72, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_1": {"addr": 73, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_2": {"addr": 74, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_2": {"addr": 75, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_2": {"addr": 76, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_2": {"addr": 77, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_2": {"addr": 78, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_2": {"addr": 79, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_2": {"addr": 80, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_2": {"addr": 81, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_2": {"addr": 82, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_2": {"addr": 83, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_2": {"addr": 84, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_2": {"addr": 85, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_2": {"addr": 86, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_2": {"addr": 87, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_2": {"addr": 88, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_3": {"addr": 89, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_3": {"addr": 90, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_3": {"addr": 91, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_3": {"addr": 92, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_3": {"addr": 93, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_3": {"addr": 94, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_3": {"addr": 95, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_3": {"addr": 96, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_3": {"addr": 97, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_3": {"addr": 98, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_3": {"addr": 99, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_3": {"addr": 100, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_3": {"addr": 101, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_3": {"addr": 102, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_3": {"addr": 103, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_4": {"addr": 104, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_4": {"addr": 105, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_4": {"addr": 106, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_4": {"addr": 107, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_4": {"addr": 108, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_4": {"addr": 109, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_4": {"addr": 110, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_4": {"addr": 111, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_4": {"addr": 112, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_4": {"addr": 113, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_4": {"addr": 114, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_4": {"addr": 115, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_4": {"addr": 116, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_4": {"addr": 117, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_4": {"addr": 118, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 119, "format": "I", "size": 4, "egu": ""},
}

bsmp_2p2s = {
    "i_load_mean": {"addr": 33, "format": "f", "size": 4, "egu": "A"},
    "i_load1": {"addr": 34, "format": "f", "size": 4, "egu": "A"},
    "i_load2": {"addr": 35, "format": "f", "size": 4, "egu": "A"},
    "i_arm_1": {"addr": 36, "format": "f", "size": 4, "egu": "A"},
    "i_arm_2": {"addr": 37, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_1": {"addr": 38, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_1": {"addr": 39, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_2": {"addr": 40, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_2": {"addr": 41, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_3": {"addr": 42, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_3": {"addr": 43, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_1_4": {"addr": 44, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_4": {"addr": 45, "format": "f", "size": 4, "egu": "A"},
    "i_mod_1": {"addr": 46, "format": "f", "size": 4, "egu": "A"},
    "i_mod_2": {"addr": 47, "format": "f", "size": 4, "egu": "A"},
    "i_mod_3": {"addr": 48, "format": "f", "size": 4, "egu": "A"},
    "i_mod_4": {"addr": 49, "format": "f", "size": 4, "egu": "A"},
    "v_dclink_1": {"addr": 50, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_2": {"addr": 51, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_3": {"addr": 52, "format": "f", "size": 4, "egu": "V"},
    "v_dclink_4": {"addr": 53, "format": "f", "size": 4, "egu": "V"},
    "duty_mean": {"addr": 54, "format": "f", "size": 4, "egu": "p.u."},
    "duty_arms_diff": {"addr": 55, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_1": {"addr": 56, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_1": {"addr": 57, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_2": {"addr": 58, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_2": {"addr": 59, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_3": {"addr": 60, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_3": {"addr": 61, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_1_4": {"addr": 62, "format": "f", "size": 4, "egu": "p.u."},
    "duty_cycle_2_4": {"addr": 63, "format": "f", "size": 4, "egu": "p.u."},
    "i_leakage": {"addr": 64, "format": "f", "size": 4, "egu": "A"},
    "v_input_iib_1": {"addr": 65, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_1": {"addr": 66, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_1": {"addr": 67, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_1": {"addr": 68, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_1": {"addr": 69, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_1": {"addr": 70, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_1": {"addr": 71, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_1": {"addr": 72, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_1": {"addr": 73, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_1": {"addr": 74, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_1": {"addr": 75, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_1": {"addr": 76, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_1": {"addr": 77, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_1": {"addr": 78, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_1": {"addr": 79, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_2": {"addr": 80, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_2": {"addr": 81, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_2": {"addr": 82, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_2": {"addr": 83, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_2": {"addr": 84, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_2": {"addr": 85, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_2": {"addr": 86, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_2": {"addr": 87, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_2": {"addr": 88, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_2": {"addr": 89, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_2": {"addr": 90, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_2": {"addr": 91, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_2": {"addr": 92, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_2": {"addr": 93, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_2": {"addr": 94, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_3": {"addr": 95, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_3": {"addr": 96, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_3": {"addr": 97, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_3": {"addr": 98, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_3": {"addr": 99, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_3": {"addr": 100, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_3": {"addr": 101, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_3": {"addr": 102, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_3": {"addr": 103, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_3": {"addr": 104, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_3": {"addr": 105, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_3": {"addr": 106, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_3": {"addr": 107, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_3": {"addr": 108, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_3": {"addr": 109, "format": "I", "size": 4, "egu": ""},
    "v_input_iib_4": {"addr": 110, "format": "f", "size": 4, "egu": "V"},
    "v_output_iib_4": {"addr": 111, "format": "f", "size": 4, "egu": "V"},
    "i_igbt_1_iib_4": {"addr": 112, "format": "f", "size": 4, "egu": "A"},
    "i_igbt_2_iib_4": {"addr": 113, "format": "f", "size": 4, "egu": "A"},
    "temp_igbt_1_iib_4": {"addr": 114, "format": "f", "size": 4, "egu": "°C"},
    "temp_igbt_2_iib_4": {"addr": 115, "format": "f", "size": 4, "egu": "°C"},
    "v_driver_iib_4": {"addr": 116, "format": "f", "size": 4, "egu": "V"},
    "i_driver_1_iib_4": {"addr": 117, "format": "f", "size": 4, "egu": "A"},
    "i_driver_2_iib_4": {"addr": 118, "format": "f", "size": 4, "egu": "A"},
    "temp_inductor_iib_4": {"addr": 119, "format": "f", "size": 4, "egu": "°C"},
    "temp_heatsink_iib_4": {"addr": 120, "format": "f", "size": 4, "egu": "°C"},
    "temp_board_iib_4": {"addr": 121, "format": "f", "size": 4, "egu": "°C"},
    "rh_iib_4": {"addr": 122, "format": "f", "size": 4, "egu": "%"},
    "iib_interlocks_4": {"addr": 123, "format": "I", "size": 4, "egu": ""},
    "iib_alarms_4": {"addr": 124, "format": "I", "size": 4, "egu": ""},
    "ps_alarms": {"addr": 125, "format": "I", "size": 4, "egu": ""},
}
